/***********************************************************************
 * $Id: FXPy.cpp,v 1.5 2000/09/06 02:38:17 ljohnson Exp $
 ***********************************************************************/

#include "fx.h"
#include "FXPNGIcon.h"
#include "FXPNGImage.h"
#include "fx3d.h"
#include "Python.h"
#include "FXPyApp.h"
#include "FXPy.h"
#include "impl.h"


extern "C" char* SWIG_GetPtr(char*,void**,char*);
extern "C" char* SWIG_GetPtrObj(PyObject*, void**, char*);
extern "C" void SWIG_MakePtr(char*,void*,char*);

extern FXAPI unsigned int fxTraceLevel;

// Maps FOX objects to Python objects
static FXDict FX2PyMap;
static PyObject *pPyDict = 0;


/*
*  Call this function to stop the FOX event loop and stop
*  the program (because an exception of some kind was raised).
*  If "msg" is NULL, assume that Python has already set the
*  error object and data and just quit. If "msg" is not NULL,
*  this indicates some kind of FXPy runtime error (i.e. a
*  bug) and we need to set the error object & string for that.
*/
void raiseException(const char *msg = 0) {
	
	if (msg != 0) {
		fxTraceLevel = 11;
		FXTRACE((10, "raiseException() with msg = \"%s\"\n", msg));
		PyErr_SetString(FXPyApp::errorObject, msg);
	}
	else
		FXTRACE((10, "raiseException(NULL)\n"));
	FXPyApp::pythonExceptionRaised=TRUE;
	FXPyApp::instance()->stop(EXIT_FAILURE);
}


// Store pointer to Python dictionary
PyObject* FXPySetDict(PyObject *, PyObject *args)
{
	if (!PyArg_ParseTuple(args, "O", &pPyDict))
		return NULL;
	
	if (!PyDict_Check(pPyDict)) {
		PyErr_SetString(PyExc_TypeError,
			"FXPySetDict must have dictionary object!");
		return NULL;
	}
	Py_INCREF(Py_None);
	return Py_None;
}


// Register
void FXPyRegister(PyObject *pPyObject)
{
	FXObject *pFXObject = 0;
	SWIG_GetPtrObj(pPyObject, (void**) &pFXObject, "_FXObject_p");
	FX2PyMap.insert(FXStringFormat("%p", pFXObject).text(), pPyObject);
	Py_INCREF(pPyObject);
}


// Unregister
void FXPyUnregister(FXObject *pFXObject)
{
	PyObject *pPyObject = FXPyGetPyObject(pFXObject);
	FX2PyMap.remove(FXStringFormat("%p", pFXObject).text());
	Py_DECREF(pPyObject); // dictionary's reference
	Py_DECREF(pPyObject); // other reference
}


// Given a Python class instance which is a shadow for some
// real C++ object, return the corresponding FXObject pointer.
FXObject *FXPyGetFXObject(PyObject *pPyObject)
{
	FXObject *obj = 0;
	SWIG_GetPtrObj(pPyObject, (void**) &obj, "_FXObject_p");
	FXASSERT(obj != 0);
	return obj;
}


/*
*  If the input object has a Python shadow class instance
*  counterpart, return a new reference to that instance.
*  Otherwise return NULL.
*/
PyObject *FXPyGetPyObject(const FXObject *pFXObject)
{
	FXString s = FXStringFormat("%p", pFXObject);
	PyObject *obj = (PyObject*) FX2PyMap.find(s.text());
	if (obj != 0) {
		Py_INCREF(obj);
		return obj;
	}
	else
		return 0;
}


// Given a class name like "FXList", convert it to "_FXPyList_p".
FXString FXPyGetSWIGPtrName(const char *s)
{
	FXString classname(s);
	if (classname.left(4) != FXString("FXPy"))
		classname = FXString("FXPy") + classname.right(classname.length()-2);
	return FXStringFormat("_%s_p", classname.text());
}


// Given an FXObject, return the FXPy Python class name
FXString FXPyGetPythonClassName(const FXObject *obj)
{
	FXString pythonClassName = obj->getClassName();
	if (pythonClassName.left(4) == "FXPy")
		pythonClassName = FXString("FX") + pythonClassName.right(pythonClassName.length() - 4);
	return pythonClassName;
}


// Return a new reference to the Python shadow object for this FOX object.
// Note that this is highly dependent on SWIG1.1's shadow class implementation.
// If the passed-in pointer is NULL, or if some error occurs, the error flag
// is set and the function returns NULL.
PyObject *
FXPyMakeShadowObject(const FXObject* obj)
{
	PyObject *shadow = NULL;
	if (obj) {
		// First, determine the SWIG-mangled name for this class
		char buff[128];
		strcpy(buff, FXPyGetSWIGPtrName(obj->getClassName()).text());
		//printf("buff = \"%s\"\n", buff);

		// Next, get the SWIG pointer representation (a string)
		char swigptr[128];
		SWIG_MakePtr(swigptr, (void*) obj, buff);
		//printf("swigptr = \"%s\"\n", swigptr);
		
		// Get a pointer to the Python class object
		FXString pythonClassName = FXPyGetPythonClassName(obj) + "Ptr";
		strcpy(buff, pythonClassName.text());
		//printf("Python class name = \"%s\"\n", buff);
		PyObject *classobj = PyDict_GetItemString(pPyDict, buff);
		if (!classobj) {
			//printf("lookup for class name \"%s\" failed!\n", buff);
			return NULL;
		}

		// Construct input argument for constructor
		PyObject *args = Py_BuildValue("(s)", swigptr);
		if (!args)
			return NULL;

		shadow = PyInstance_New(classobj, args, NULL);
		if (!shadow) {
			//printf("PyInstance_New() returned NULL\n");
		}
		Py_DECREF(args);
	}
	return shadow;
}


// Return the class object for Python class FXEvent
static PyObject *
FXPyGetEventClassObject()
{
	static PyObject *EventClassObject = 0;
	if (EventClassObject == 0)
		EventClassObject = PyDict_GetItemString(pPyDict, "FXEventPtr");
	return EventClassObject;
}


// Return the class object for Python class FXTreeItem
static PyObject *
FXPyGetTreeItemClassObject()
{
	static PyObject *classObject = 0;
	if (classObject == 0)
		classObject = PyDict_GetItemString(pPyDict, "FXTreeItemPtr");
	return classObject;
}


// Returns message data appropriate for a given sender/type combination
static PyObject *
FXPyGetMessageData(FXObject *sender, FXSelector sel, void *ptr)
{
	FXASSERT(sender != 0);
	FXushort type = SELTYPE(sel);
	// FXushort id = SELID(sel);
	if (type == SEL_PAINT ||
		type == SEL_LEFTBUTTONPRESS ||
		type == SEL_LEFTBUTTONRELEASE ||
		type == SEL_MOTION) {
		// Data is pointer to the FXEvent record
		char swigptr[128];
		SWIG_MakePtr(swigptr, ptr, "_FXEvent_p");
		PyObject *eventClass = FXPyGetEventClassObject();
		PyObject *args = Py_BuildValue("(s)", swigptr);
		PyObject *data = PyInstance_New(eventClass, args, NULL);
		Py_DECREF(args);
		return data;
	}
	else if (sender->isMemberOf(FXMETACLASS(FXHeader))) {
		FXint active = (FXint) ptr;
		if (type == SEL_COMMAND || type == SEL_CHANGED)
			return Py_BuildValue("i", active);
	}
	else if (sender->isMemberOf(FXMETACLASS(FXRecentFiles))) {
		FXchar *filename = (FXchar*) ptr;
		if (type == SEL_COMMAND)
			return Py_BuildValue("s", filename);
	}
	else if (sender->isMemberOf(FXMETACLASS(FXTreeList))) {
		// FXTreeItem *item = (FXTreeItem*) ptr;
		if (type == SEL_COMMAND || type == SEL_CHANGED) {
			char swigptr[128];
			SWIG_MakePtr(swigptr, ptr, "_FXPyTreeItem_p");
			PyObject *eventClass = FXPyGetTreeItemClassObject();
			PyObject *args = Py_BuildValue("(s)", swigptr);
			PyObject *data = PyInstance_New(eventClass, args, NULL);
			Py_DECREF(args);
			return data;
		}
	}

	if (ptr == 0) {
		Py_INCREF(Py_None);
		return Py_None;
	}
	else {
		char s[256];
		sprintf(s, "%p", ptr);
		return PyString_FromString(s);
	}
}


// Check for handle
FXbool FXPyCanHandle(FXObject* self, FXSelector sel)
{
	FXbool result = FALSE;
	PyObject* pPyObject = FXPyGetPyObject(self);
	if (pPyObject) {
		if (PyObject_HasAttrString(pPyObject, "_search")) {
			PyObject *pMethObj = PyObject_CallMethod(pPyObject, "_search",
				"(i)", sel);
			Py_DECREF(pPyObject);
			if (pMethObj) {
				if (pMethObj == Py_None) {
					Py_DECREF(pMethObj);
					result = FALSE;
				}
				else {
					Py_DECREF(pMethObj);
					result = TRUE;
				}
			}
		}
		else {
			Py_DECREF(pPyObject);
			raiseException("object didn't define an _search() method");
		}
	}
	return result;
}


// Handle message
long FXPyHandle(FXObject* self, FXObject* sender, FXSelector sel, void* ptr)
{
	// Get Python object for this C++ object
	PyObject *pPyObject = FXPyGetPyObject(self);
	if (pPyObject) {
		// Call _search() to get the method associated with this selector
		PyObject *pMethObj = PyObject_CallMethod(pPyObject, "_search", "(i)", sel);
		if (!pMethObj) {
			Py_DECREF(pPyObject);
			raiseException();
			return 0;
		}
		
		// If _search() returns None then we don't want to handle this one
		if (pMethObj == Py_None) {
			Py_DECREF(Py_None);
			Py_DECREF(pPyObject);
			return 0;
		}
		
		// Make sure it's really a callable object
		if (!PyCallable_Check(pMethObj)) {
			PyErr_SetString(PyExc_TypeError, "not a callable object");
			Py_DECREF(pMethObj);
			Py_DECREF(pPyObject);
			raiseException();
			return 0;
		}
		
		// The sender may or may not have a Python counterpart, depending
		// on whether it was created in Python code or its some kind of
		// object created by FOX itself.
		PyObject *pSender = FXPyGetPyObject(sender);
		if (!pSender)
			pSender = FXPyMakeShadowObject(sender);

		if (pSender) {
			// Convert message data to something usable in Python
			PyObject *data = FXPyGetMessageData(sender, sel, ptr);

			// Call the message handler function (an unbound method)
			PyObject *pResObj = PyObject_CallFunction(pMethObj, "OOiO",
				pPyObject, pSender, sel, data);

			Py_DECREF(data);
			Py_DECREF(pMethObj);
			Py_DECREF(pPyObject);
			
			// Message handler must return an integer value
			if (pResObj) {
				if (PyInt_Check(pResObj)) {
					long nRes = PyInt_AsLong(pResObj);
					Py_DECREF(pResObj);
					return nRes;
				}
				else {
					Py_DECREF(pResObj);
					raiseException("message handler function must return an integer");
					return 1;
				}
			}
			else {
				raiseException();
				return 1;
			}
		}
		else {
			Py_DECREF(pPyObject);
			Py_DECREF(pMethObj);
			raiseException();
			return 0;
		}
	}
	return 1;
}


// Call a method named "func" of the shadow class "obj".
// The method is assumed to return None.
static void FXPyCallVoidFunction(FXObject *obj, char *func)
{
	// Lookup shadow object pointer
	PyObject *pPyObject = FXPyGetPyObject(obj);

	if (pPyObject) {
		// Call the function
		PyObject *result = PyObject_CallMethod(pPyObject, func, NULL);
		Py_DECREF(pPyObject);
		
		// Function is expected to return None
		if (result) {
			if (result == Py_None) {
				Py_DECREF(Py_None);
				return;
			}
			else {
				Py_DECREF(result);
				PyErr_SetString(PyExc_TypeError,
					"Expected function to return None");
				raiseException();
			}
		}
		else {
			raiseException();
		}
	}
}


// Call a method named "func" of the shadow class "obj", passing
// it the shadow class instance for "arg1" as its single argument.
// The method is assumed to return None.
static void FXPyCallVoidFunction(FXObject *obj, char *func, PyObject *arg1)
{
	// Lookup shadow object pointer
	PyObject *pPyObject = FXPyGetPyObject(obj);

	if (pPyObject) {
		// Call the function
		PyObject *result = PyObject_CallMethod(pPyObject, func,
			"(O)", arg1);
		Py_DECREF(pPyObject);
		
		// Function is expected to return None
		if (result) {
			if (result == Py_None) {
				Py_DECREF(Py_None);
				return;
			}
			else {
				Py_DECREF(result);
				PyErr_SetString(PyExc_TypeError,
					"Expected function to return None");
				raiseException();
			}
		}
		else {
			raiseException();
		}
	}
}

static void
FXPyCallVoidFunction(FXObject *obj, char *func, FXObject* arg1)
{
	PyObject *pyarg1 = FXPyGetPyObject(arg1);
	FXPyCallVoidFunction(obj, func, pyarg1);
	Py_DECREF(pyarg1);
}

static void
FXPyCallVoidFunction(FXObject *obj, char *func,
		     PyObject *arg1, PyObject *arg2)
{
	// Lookup shadow object pointer
	PyObject *pPyObject = FXPyGetPyObject(obj);

	if (pPyObject) {
		// Call the function
		PyObject *result = PyObject_CallMethod(pPyObject, func, "(OO)",
			arg1, arg2);
		Py_DECREF(pPyObject);
		
		// Function is expected to return None
		if (result) {
			if (result == Py_None) {
				Py_DECREF(Py_None);
				return;
			}
			else {
				Py_DECREF(result);
				PyErr_SetString(PyExc_TypeError,
					"Expected function to return None");
				raiseException();
			}
		}
		else {
			raiseException();
		}
	}
}


static void
FXPyCallVoidFunction(FXObject *obj, char *func,
		     PyObject *arg1, PyObject *arg2,
		     PyObject *arg3, PyObject *arg4)
{
	// Lookup shadow object pointer
	PyObject *pPyObject = FXPyGetPyObject(obj);
	if (pPyObject) {
		// Call the function
		PyObject *result = PyObject_CallMethod(pPyObject, func,
			"(OOOO)", arg1, arg2, arg3, arg4);
		Py_DECREF(pPyObject);
		
		// Function is expected to return None
		if (result) {
			if (result == Py_None) {
				Py_DECREF(Py_None);
				return;
			}
			else {
				Py_DECREF(result);
				PyErr_SetString(PyExc_TypeError,
					"Expected function to return None");
				raiseException();
			}
		}
		else {
			raiseException();
		}
	}
}


// Call a method named "func" of the shadow class "obj".
// The method is assumed to return a boolean value.
static FXbool FXPyCallBoolFunction(const FXObject* obj, char *func)
{
	PyObject *pPyObject = FXPyGetPyObject(obj);
	if (pPyObject) {
		PyObject *result = PyObject_CallMethod(pPyObject, func, NULL);
		Py_DECREF(pPyObject);
		if (result) {
			if (PyInt_Check(result)) {
				FXbool bRes = (FXbool) PyInt_AsLong(result);
				Py_DECREF(result);
				return bRes;
			}
			else {
				Py_DECREF(result);
				raiseException("Expected function to return 0 or 1");
			}
		}
		else {
			raiseException();
		}
	}
	return FALSE;
}


// Call a method named "func" of the shadow class "obj".
// The method is assumed to return an FXint value.
static FXint FXPyCallIntFunction(FXObject* obj, char *func, PyObject *arg1)
{
	PyObject *pPyObject = FXPyGetPyObject(obj);
	if (pPyObject) {
		PyObject *result = PyObject_CallMethod(pPyObject, func, "(O)",
			arg1);
		Py_DECREF(pPyObject);
		if (result) {
			if (PyInt_Check(result)) {
				FXint nResult = (FXint) PyInt_AsLong(result);
				Py_DECREF(result);
				return nResult;
			}
			else {
				Py_DECREF(result);
				raiseException("Expected function to return an integer");
			}
		}
		else {
			raiseException();
		}
	}
	return -1;
}


void FXPyCreate(FXObject *obj) { FXPyCallVoidFunction(obj, "create"); }
void FXPyDetach(FXObject *obj) { FXPyCallVoidFunction(obj, "detach"); }
void FXPyDestroy(FXObject *obj) { FXPyCallVoidFunction(obj, "destroy"); }
void FXPyShow(FXObject *obj) { FXPyCallVoidFunction(obj, "show"); }


// long FXObject::onDefault(FXObject* sender,FXSelector sel,void* ptr)
static long
_onDefault(FXObject* fxObj,FXObject* sender,FXSelector sel,void* ptr) {
	PyObject *pyObj = FXPyGetPyObject(fxObj);
	if (pyObj != NULL) {
		PyObject *arg0 = FXPyMakeShadowObject(sender);
		if (arg0 == NULL) {
			Py_DECREF(pyObj);
			raiseException();
			return 0;
		}
		PyObject *result = PyObject_CallMethod(pyObj, "onDefault", "(Oii)", arg0, sel, 0);
		Py_DECREF(arg0);
		Py_DECREF(pyObj);
		if (result != NULL) {
			if (PyInt_Check(result)) {
				long longResult = PyInt_AsLong(result);
				Py_DECREF(result);
				return longResult;
			}
			else {
				Py_DECREF(result);
				raiseException("onDefault() should return an integer");
			}
			Py_DECREF(result);
		}
		else {
			raiseException();
		}
	}
	return 0;
}


// FXWindow::getDefaultWidth() const
static FXint
_getDefaultWidth(const FXWindow* win) {
	PyObject *obj = FXPyGetPyObject(win);
	if (obj) {
		PyObject *result = PyObject_CallMethod(obj, "getDefaultWidth", NULL);
		Py_DECREF(obj);
		if (result) {
			if (PyInt_Check(result)) {
				FXint defaultWidth = PyInt_AsLong(result);
				FXASSERT(defaultWidth >= 0);
				Py_DECREF(result);
				return defaultWidth;
			}
			else {
				Py_DECREF(result);
				raiseException("getDefaultWidth() should return an integer");
			}
		}
		else {
			raiseException();
		}
	}
	return -1;
}


// FXWindow::getDefaultHeight() const
static FXint _getDefaultHeight(const FXWindow* win) {
	PyObject *obj = FXPyGetPyObject(win);
	if (obj) {
		PyObject *result = PyObject_CallMethod(obj, "getDefaultHeight", NULL);
		Py_DECREF(obj);
		if (result) {
			if (PyInt_Check(result)) {
				FXint defaultHeight = PyInt_AsLong(result);
				FXASSERT(defaultHeight >= 0);
				Py_DECREF(result);
				return defaultHeight;
			}
			else {
				Py_DECREF(result);
				raiseException("getDefaultHeight() should return an integer");
			}
		}
		else {
			raiseException();
		}
	}
	return -1;
}


// FXWindow::layout()
static void _layout(FXWindow* win) { FXPyCallVoidFunction(win, "layout"); }


// FXint FXTreeItem::getWidth(const FXTreeList* list) const
static FXint _getWidth(const FXTreeItem *self, const FXTreeList *list)
{
	PyObject *obj = FXPyGetPyObject(self);
	if (obj) {
		PyObject *arg1 = FXPyMakeShadowObject(list);
		if (!arg1) {
			Py_DECREF(obj);
			raiseException();
			return -1;
		}
		PyObject *result = PyObject_CallMethod(obj, "getWidth", "(O)", arg1);
		Py_DECREF(arg1);
		Py_DECREF(obj);
		if (result) {
			if (PyInt_Check(result)) {
				FXint w = PyInt_AsLong(result);
				Py_DECREF(result);
				return w;
			}
			else {
				Py_DECREF(result);
				raiseException("getWidth() should return an integer");
				return -1;
			}
		}
		else
			raiseException();
	}
	return -1;
}


// FXint FXTreeItem::getHeight(const FXTreeList* list) const
static FXint _getHeight(const FXTreeItem *self, const FXTreeList *list)
{
	PyObject *obj = FXPyGetPyObject(self);
	if (obj) {
		PyObject *arg1 = FXPyMakeShadowObject(list);
		if (!arg1) {
			Py_DECREF(obj);
			raiseException();
			return -1;
		}
		PyObject *result = PyObject_CallMethod(obj, "getHeight", "(O)", arg1);
		Py_DECREF(arg1);
		Py_DECREF(obj);
		if (result) {
			if (PyInt_Check(result)) {
				FXint h = PyInt_AsLong(result);
				Py_DECREF(result);
				return h;
			}
			else {
				Py_DECREF(result);
				raiseException("getHeight() should return an integer");
				return -1;
			}
		}
		else
			raiseException();
	}
	return -1;
}


// FXint FXHeaderItem::getWidth(const FXHeader* header) const
static FXint _getWidth(const FXHeaderItem *self, const FXHeader *header)
{
	PyObject *obj = FXPyGetPyObject(self);
	if (obj) {
		PyObject *arg1 = FXPyMakeShadowObject(header);
		if (!arg1) {
			Py_DECREF(obj);
			raiseException();
			return -1;
		}
		PyObject *result = PyObject_CallMethod(obj, "getWidth", "(O)", arg1);
		Py_DECREF(arg1);
		Py_DECREF(obj);
		if (result) {
			if (PyInt_Check(result)) {
				FXint w = PyInt_AsLong(result);
				Py_DECREF(result);
				return w;
			}
			else {
				Py_DECREF(result);
				raiseException("getWidth() should return an integer");
				return -1;
			}
		}
		else
			raiseException();
	}
	return -1;
}


// FXint FXHeaderItem::getHeight(const FXHeader* header) const
static FXint _getHeight(const FXHeaderItem *self, const FXHeader *header)
{
	PyObject *obj = FXPyGetPyObject(self);
	if (obj) {
		PyObject *arg1 = FXPyMakeShadowObject(header);
		if (!arg1) {
			Py_DECREF(obj);
			raiseException();
			return -1;
		}
		PyObject *result = PyObject_CallMethod(obj, "getHeight", "(O)", arg1);
		Py_DECREF(arg1);
		Py_DECREF(obj);
		if (result) {
			if (PyInt_Check(result)) {
				FXint h = PyInt_AsLong(result);
				Py_DECREF(result);
				return h;
			}
			else {
				Py_DECREF(result);
				raiseException("getHeight() should return an integer");
				return -1;
			}
		}
		else
			raiseException();
	}
	return -1;
}


// FXint FXIconItem::getWidth(const FXIconList* list) const
static FXint _getWidth(const FXIconItem *self, const FXIconList *list)
{
	PyObject *obj = FXPyGetPyObject(self);
	if (obj) {
		PyObject *arg1 = FXPyMakeShadowObject(list);
		if (!arg1) {
			Py_DECREF(obj);
			raiseException();
			return -1;
		}
		PyObject *result = PyObject_CallMethod(obj, "getWidth", "(O)", arg1);
		Py_DECREF(arg1);
		Py_DECREF(obj);
		if (result) {
			if (PyInt_Check(result)) {
				FXint w = PyInt_AsLong(result);
				Py_DECREF(result);
				return w;
			}
			else {
				Py_DECREF(result);
				raiseException("getWidth() should return an integer");
				return -1;
			}
		}
		else
			raiseException();
	}
	return -1;
}


// FXint FXIconItem::getHeight(const FXIconList* list) const
static FXint _getHeight(const FXIconItem *self, const FXIconList *list)
{
	PyObject *obj = FXPyGetPyObject(self);
	if (obj) {
		PyObject *arg1 = FXPyMakeShadowObject(list);
		if (!arg1) {
			Py_DECREF(obj);
			raiseException();
			return -1;
		}
		PyObject *result = PyObject_CallMethod(obj, "getHeight", "(O)", arg1);
		Py_DECREF(arg1);
		Py_DECREF(obj);
		if (result) {
			if (PyInt_Check(result)) {
				FXint h = PyInt_AsLong(result);
				Py_DECREF(result);
				return h;
			}
			else {
				Py_DECREF(result);
				raiseException("getHeight() should return an integer");
				return -1;
			}
		}
		else
			raiseException();
	}
	return -1;
}


// FXint FXListItem::getWidth(const FXList* list) const
static FXint _getWidth(const FXListItem *self, const FXList *list)
{
	PyObject *obj = FXPyGetPyObject(self);
	if (obj) {
		PyObject *arg1 = FXPyMakeShadowObject(list);
		if (!arg1) {
			Py_DECREF(obj);
			raiseException();
			return -1;
		}
		PyObject *result = PyObject_CallMethod(obj, "getWidth", "(O)", arg1);
		Py_DECREF(arg1);
		Py_DECREF(obj);
		if (result) {
			if (PyInt_Check(result)) {
				FXint w = PyInt_AsLong(result);
				Py_DECREF(result);
				return w;
			}
			else {
				Py_DECREF(result);
				raiseException("getWidth() should return an integer");
				return -1;
			}
		}
		else
			raiseException();
	}
	return -1;
}


// FXint FXListItem::getHeight(const FXList* list) const
static FXint _getHeight(const FXListItem *self, const FXList *list)
{
	PyObject *obj = FXPyGetPyObject(self);
	if (obj) {
		PyObject *arg1 = FXPyMakeShadowObject(list);
		if (!arg1) {
			Py_DECREF(obj);
			raiseException();
			return -1;
		}
		PyObject *result = PyObject_CallMethod(obj, "getHeight", "(O)", arg1);
		Py_DECREF(arg1);
		Py_DECREF(obj);
		if (result) {
			if (PyInt_Check(result)) {
				FXint h = PyInt_AsLong(result);
				Py_DECREF(result);
				return h;
			}
			else {
				Py_DECREF(result);
				raiseException("getHeight() should return an integer");
				return -1;
			}
		}
		else
			raiseException();
	}
	return -1;
}


// FXint FXTableItem::getWidth(const FXTable* table) const
static FXint _getWidth(const FXTableItem *self, const FXTable *table)
{
	PyObject *obj = FXPyGetPyObject(self);
	if (obj) {
		PyObject *arg1 = FXPyMakeShadowObject(table);
		if (!arg1) {
			Py_DECREF(obj);
			raiseException();
			return -1;
		}
		PyObject *result = PyObject_CallMethod(obj, "getWidth", "(O)", arg1);
		Py_DECREF(arg1);
		Py_DECREF(obj);
		if (result) {
			if (PyInt_Check(result)) {
				FXint w = PyInt_AsLong(result);
				Py_DECREF(result);
				return w;
			}
			else {
				Py_DECREF(result);
				raiseException("getWidth() should return an integer");
				return -1;
			}
		}
		else
			raiseException();
	}
	return -1;
}


// FXint FXTableItem::getHeight(const FXTable* table) const
static FXint _getHeight(const FXTableItem *self, const FXTable *table)
{
	PyObject *obj = FXPyGetPyObject(self);
	if (obj) {
		PyObject *arg1 = FXPyMakeShadowObject(table);
		if (!arg1) {
			Py_DECREF(obj);
			raiseException();
			return -1;
		}
		PyObject *result = PyObject_CallMethod(obj, "getHeight", "(O)", arg1);
		Py_DECREF(arg1);
		Py_DECREF(obj);
		if (result) {
			if (PyInt_Check(result)) {
				FXint h = PyInt_AsLong(result);
				Py_DECREF(result);
				return h;
			}
			else {
				Py_DECREF(result);
				raiseException("getHeight() should return an integer");
				return -1;
			}
		}
		else
			raiseException();
	}
	return -1;
}


// void FXGLObject::draw(FXGLViewer* viewer)
static void _draw(FXGLObject* self, FXGLViewer* viewer) {
	FXPyCallVoidFunction(self, "draw", viewer);
}


// void FXGLObject::hit(FXGLViewer* viewer)
static void _hit(FXGLObject* self, FXGLViewer* viewer) {
	FXPyCallVoidFunction(self, "hit", viewer);
}


// FXbool FXGLObject::canDrag()
static FXbool _canDrag(const FXGLObject* self) {
	return FXPyCallBoolFunction(self, "canDrag");
}


// FXbool FXGLObject::canDelete()
static FXbool _canDelete(const FXGLObject* self) {
	return FXPyCallBoolFunction(self, "canDelete");
}


// FXbool FXGLObject::drag(FXGLViewer* viewer, FXint fx, FXint fy, FXint tx, FXint ty)
static FXbool
_drag(FXGLObject* self, FXGLViewer* viewer, FXint fx, FXint fy, FXint tx, FXint ty) {
	PyObject *obj = FXPyGetPyObject(self);
	if (obj) {
		PyObject *v = FXPyGetPyObject(viewer);
		if (!v) {
			Py_DECREF(obj);
			raiseException();
			return FALSE;
		}

		PyObject *result = PyObject_CallMethod(obj, "drag",
			"(Oiiii)", v, fx, fy, tx, ty);
		Py_DECREF(v);
		Py_DECREF(obj);

		if (result) {
			if (PyInt_Check(result)) {
				FXbool bRes = (FXbool) PyInt_AsLong(result);
				Py_DECREF(result);
				return bRes;
			}
			else {
				Py_DECREF(result);
				raiseException("drag() should return either zero or one");
				return FALSE;
			}
		}
		else {
			raiseException();
			return FALSE;
		}
	}
	return FALSE;
}



static void _setText(FXObject *self, const FXString& text)
{
	PyObject *s = PyString_FromString(text.text());
	if (s) {
		FXPyCallVoidFunction(self, "setText", s);
		Py_DECREF(s);
	}
	else
		raiseException();
}


static void _setIcon(FXObject *self, FXIcon *icn) {
    FXPyCallVoidFunction(self, "setIcon", icn);
}


static void _setMiniIcon(FXIconItem *self, FXIcon *icn) {
    FXPyCallVoidFunction(self, "setMiniIcon", icn);
}


static void _setBigIcon(FXIconItem *self, FXIcon *icn) {
    FXPyCallVoidFunction(self, "setBigIcon", icn);
}


static void _setOpenIcon(FXTreeItem *self, FXIcon *icn) {
    FXPyCallVoidFunction(self, "setOpenIcon", icn);
}


static void _setClosedIcon(FXTreeItem *self, FXIcon *icn) {
    FXPyCallVoidFunction(self, "setClosedIcon", icn);
}


static void _setFocus(FXObject *self, FXbool focus) {
    PyObject *i = PyInt_FromLong((long) focus);
    if (i) {
	FXPyCallVoidFunction(self, "setFocus", i);
	Py_DECREF(i);
    }
    else
	    raiseException();
}

static void _setSelected(FXObject *self, FXbool selected) {
    PyObject *i = PyInt_FromLong((long) selected);
    if (i) {
	FXPyCallVoidFunction(self, "setSelected", i);
	Py_DECREF(i);
    }
    else
	    raiseException();
}

static void _setOpened(FXTreeItem *self, FXbool opened) {
    PyObject *i = PyInt_FromLong((long) opened);
    if (i) {
	FXPyCallVoidFunction(self, "setOpened", i);
	Py_DECREF(i);
    }
    else
	    raiseException();
}

static void _setExpanded(FXTreeItem *self, FXbool expanded) {
    PyObject *i = PyInt_FromLong((long) expanded);
    if (i) {
	FXPyCallVoidFunction(self, "setExpanded", i);
	Py_DECREF(i);
    }
    else
	    raiseException();
}

static void _setEnabled(FXObject *self, FXbool enabled) {
    PyObject *i = PyInt_FromLong((long) enabled);
    if (i) {
	FXPyCallVoidFunction(self, "setEnabled", i);
	Py_DECREF(i);
    }
    else
	    raiseException();
}

static void _setDraggable(FXObject *self, FXbool draggable) {
    PyObject *i = PyInt_FromLong((long) draggable);
    if (i) {
	FXPyCallVoidFunction(self, "setDraggable", i);
	Py_DECREF(i);
    }
    else
	    raiseException();
}

static void _setMarked(FXObject *self, FXbool marked) {
    PyObject *i = PyInt_FromLong((long) marked);
    if (i) {
	FXPyCallVoidFunction(self, "setMarked", i);
	Py_DECREF(i);
    }
    else
	    raiseException();
}

static void _setJustify(FXTableItem *self, FXuint justify) {
    PyObject *i = PyInt_FromLong((long) justify);
    if (i) {
	FXPyCallVoidFunction(self, "setJustify", i);
	Py_DECREF(i);
    }
    else
	    raiseException();
}


// void FXWindow::enable()
static void _enable(FXWindow *self) {
    FXPyCallVoidFunction(self, "enable");
}


// void FXWindow::disable()
static void _disable(FXWindow *self) {
    FXPyCallVoidFunction(self, "disable");
}


// void FXWindow::hide()
static void _hide(FXWindow *self) {
    FXPyCallVoidFunction(self, "hide");
}


// void FXWindow::lower()
static void _lowerWindow(FXWindow *self) {
    FXPyCallVoidFunction(self, "lower");
}


// FXint FXWindow::getWidthForHeight(FXint h)
static FXint _getWidthForHeight(FXWindow *self, FXint h) {
    PyObject *o = PyInt_FromLong((long) h);
    if (o) {
	FXint w = FXPyCallIntFunction(self, "getWidthForHeight", o);
	Py_DECREF(o);
	return w;
    }
    else {
	    raiseException();
	    return -1;
    }
    
}


// FXint FXWindow::getHeightForWidth(FXint w)
static FXint _getHeightForWidth(FXWindow *self, FXint w) {
    PyObject *o = PyInt_FromLong((long) w);
    if (o) {
	FXint h = FXPyCallIntFunction(self, "getHeightForWidth", o);
	Py_DECREF(o);
	return h;
    }
    else {
	    raiseException();
	    return -1;
    }
}


// void FXWindow::recalc()
static void _recalc(FXWindow *self) {
    FXPyCallVoidFunction(self, "recalc");
}


// FXbool FXWindow::isComposite() const
static FXbool _isComposite(const FXWindow *self) {
    return FXPyCallBoolFunction(self, "isComposite");
}


// void FXWindow::move(FXint x, FXint y)
static void _move(FXWindow *self, FXint x, FXint y) {
	PyObject *arg1 = PyInt_FromLong(x);
	PyObject *arg2 = PyInt_FromLong(y);
	if (arg1 && arg2) {
		FXPyCallVoidFunction(self, "move", arg1, arg2);
		Py_DECREF(arg2);
		Py_DECREF(arg1);
	}
	else {
		Py_XDECREF(arg1);
		Py_XDECREF(arg2);
		raiseException();
	}
}


// void FXWindow::resize(FXint w, FXint h)
static void _resize(FXWindow *self, FXint w, FXint h) {
	PyObject *arg1 = PyInt_FromLong(w);
	PyObject *arg2 = PyInt_FromLong(h);
	if (arg1 && arg2) {
		FXPyCallVoidFunction(self, "resize", arg1, arg2);
		Py_DECREF(arg2);
		Py_DECREF(arg1);
	}
	else {
		Py_XDECREF(arg1);
		Py_XDECREF(arg2);
		raiseException();
	}
}


// void FXWindow::position(FXint x, FXint y, FXint w, FXint h)
static void _position(FXWindow *self, FXint x, FXint y, FXint w, FXint h) {
	PyObject *arg1 = PyInt_FromLong(x);
	PyObject *arg2 = PyInt_FromLong(y);
	PyObject *arg3 = PyInt_FromLong(w);
	PyObject *arg4 = PyInt_FromLong(h);
	if (arg1 && arg2 && arg3 && arg4) {
		FXPyCallVoidFunction(self, "position", arg1, arg2, arg3, arg4);
		Py_DECREF(arg4);
		Py_DECREF(arg3);
		Py_DECREF(arg2);
		Py_DECREF(arg1);
	}
	else {
		Py_XDECREF(arg1);
		Py_XDECREF(arg2);
		Py_XDECREF(arg3);
		Py_XDECREF(arg4);
	}
}


// FXbool FXWindow::canFocus()
static FXbool _canFocus(const FXWindow *self) {
    return FXPyCallBoolFunction(self, "canFocus");
}


// void FXWindow::setFocus()
static void _setFocus(FXWindow *self) {
    FXPyCallVoidFunction(self, "setFocus");
}


// void FXWindow::killFocus()
static void _killFocus(FXWindow *self) {
    FXPyCallVoidFunction(self, "killFocus");
}


// FXbool FXWindow::contains(FXint x, FXint y)
static FXbool _contains(const FXWindow *self, FXint x, FXint y) {
	PyObject *obj = FXPyGetPyObject(self);
	if (obj) {
		PyObject *result = PyObject_CallMethod(obj, "contains",
			"(ii)", x, y);
		Py_DECREF(obj);
		if (result) {
			if (PyInt_Check(result)) {
				FXbool bRes = (FXbool) PyInt_AsLong(result);
				Py_DECREF(result);
				return bRes;
			}
			else {
				Py_DECREF(result);
				raiseException("contains() should return either zero or one");
				return FALSE;
			}
		}
		else {
			raiseException();
			return FALSE;
		}
	}
	else {
		raiseException();
		return FALSE;
	}
}

#include "virtuals.cpp"
