/***********************************************************************
 * $Id: FXPy.h,v 1.5 2000/09/06 02:38:17 ljohnson Exp $
 ***********************************************************************/

#ifndef FXPY_H
#define FXPY_H


// Global functions
extern FXbool		FXPyCanHandle(FXObject* self,FXSelector sel);
extern void		FXPyCreate(FXObject* obj);
extern void		FXPyDetach(FXObject* obj);
extern void		FXPyDestroy(FXObject* obj);
extern void		FXPyShow(FXObject* obj);
extern long		FXPyHandle(FXObject* self, FXObject* sender,
				   FXSelector sel, void* ptr);
extern void		FXPyRegister(PyObject *pPyObject);
extern void		FXPyUnregister(FXObject *pFXObject);
extern FXObject*	FXPyGetFXObject(PyObject *pPyObject);
extern PyObject*	FXPyGetPyObject(const FXObject *pFXObject);
extern PyObject*	FXPySetDict(PyObject* self, PyObject* args);
extern PyObject*	FXPyMakeShadowObject(const FXObject* obj);


/**
 * Macro to set up class implementation.
 * This is a modified version of the standard FOX macro FXIMPLEMENT();
 * this version's implementation of the virtual handle() function
 * first checks the Python class instance's message map to see if it
 * can handle the incoming message. If not, the regular C++ message
 * mapping takes over as in standard FOX applications.
 */

#define FXPy_IMPLEMENT(classname,baseclassname,mapping,nmappings) \
  FXObject* classname::manufacture(){return new classname;} \
  const FXMetaClass classname::metaClass={#classname,classname::manufacture,&baseclassname::metaClass,mapping,nmappings,sizeof(classname::FXMapEntry),sizeof(#classname)}; \
  __FXMETACLASSINITIALIZER__ classname##Initializer(&classname::metaClass); \
  long classname::handle(FXObject* sender,FXSelector key,void* data){ \
    if(FXPyCanHandle(this,key)){ \
      return FXPyHandle(this,sender,key,data); \
      } \
    else { \
      const FXMapEntry* me=(const FXMapEntry*)metaClass.search(key); \
      return me ? (this->* me->func)(sender,key,data) : baseclassname::handle(sender,key,data); \
      } \
    } 


// Forward declarations as needed
class FXPyApp;
class FXPyGLViewer;


// Base class for FOX objects
class FXPyObject : public FXObject {
  FXDECLARE(FXPyObject)
#include "object_virtuals.h"
  };


// Accelerator table
class FXPyAccelTable : public FXAccelTable {
  FXDECLARE(FXPyAccelTable)
public:
  FXPyAccelTable(){}
#include "object_virtuals.h"
  };


// Handle to something in the server
class FXPyId : public FXId {
  FXDECLARE(FXPyId)
protected:
  FXPyId(){}
public:
  FXPyId(FXPyApp* app) : FXId(app){}
#include "object_virtuals.h"
  };


// Cursor
class FXPyCursor : public FXCursor {
  FXDECLARE(FXPyCursor)
protected:
  FXPyCursor(){}
  FXPyCursor(const FXPyCursor&);
public:
  FXPyCursor(FXPyApp* app,const void* src,const void* msk,FXint w,FXint h,FXint hx,FXint hy) : FXCursor(app,src,msk,w,h,hx,hy){}
#include "object_virtuals.h"
  };


// Icon
class FXPyIcon : public FXIcon {
  FXDECLARE(FXPyIcon)
protected:
  FXPyIcon(){}
public:
  FXPyIcon(FXPyApp* a,const void *pix,FXColor clr,FXuint opts,FXint w,FXint h) : FXIcon(a,pix,clr,opts,w,h){}
#include "object_virtuals.h"
  };


// GIF icon
class FXPyGIFIcon : public FXGIFIcon {
  FXDECLARE(FXPyGIFIcon)
protected:
  FXPyGIFIcon(){}
public:
  FXPyGIFIcon(FXPyApp* app,const void* pix,FXColor clr,FXuint opts,FXint w,FXint h) : FXGIFIcon(app,pix,clr,opts,w,h){}
#include "object_virtuals.h"
  };


// BMP icon
class FXPyBMPIcon : public FXBMPIcon {
  FXDECLARE(FXPyBMPIcon)
protected:
  FXPyBMPIcon(){}
  FXPyBMPIcon(const FXPyBMPIcon&);
public:
  FXPyBMPIcon(FXPyApp* app,const void* pix,FXColor clr,FXuint opts,FXint w,FXint h) : FXBMPIcon(app,pix,clr,opts,w,h){}
#include "object_virtuals.h"
  };


// PNG icon
class FXPyPNGIcon : public FXPNGIcon {
  FXDECLARE(FXPyPNGIcon)
protected:
  FXPyPNGIcon(){}
  FXPyPNGIcon(const FXPyPNGIcon&);
public:
  FXPyPNGIcon(FXPyApp* app,const void* pix,FXColor clr,FXuint opts,FXint w,FXint h) : FXPNGIcon(app,pix,clr,opts,w,h){}
#include "object_virtuals.h"
  };


// Font
class FXPyFont : public FXFont {
  FXDECLARE(FXPyFont)
protected:
  FXPyFont(){}
public:
  FXPyFont(FXPyApp* a,const FXString& text) : FXFont(a,text) {}
  FXPyFont(FXApp* a,const FXString& face,FXuint sz,FXuint wt,FXuint sl,FXuint enc,FXuint setw,FXuint h) : FXFont(a,face,sz,wt,sl,enc,setw,h) {}
  FXPyFont(FXApp* a,const FXFontDesc& fontdesc) : FXFont(a,fontdesc) {}
#include "object_virtuals.h"
  };


// Composite
class FXPyComposite : public FXComposite {
  FXDECLARE(FXPyComposite)
protected:
  FXPyComposite(){}
public:
  FXPyComposite(FXPyComposite* p,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0) : FXComposite(p,opts,x,y,w,h){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Button
class FXPyButton : public FXButton {
  FXDECLARE(FXPyButton)
protected:
  FXPyButton(){}
public:
  FXPyButton(FXPyComposite* p,const FXString& text,FXPyIcon *ic=0,FXPyObject* tgt=NULL,FXSelector sel=0,FXuint opts=JUSTIFY_NORMAL|ICON_BEFORE_TEXT|FRAME_RAISED|FRAME_THICK,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pb=DEFAULT_PAD,FXint pt=DEFAULT_PAD) : FXButton(p,text,ic,tgt,sel,opts,x,y,w,h,pl,pr,pb,pt){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };

// Device Context
class FXPyDC : public FXDC {
public:
  FXPyDC(FXPyApp* app) : FXDC(app){}
  };

// Window Device Context
class FXPyDCWindow : public FXDCWindow {
public:
  FXPyDCWindow(FXDrawable* drawable) : FXDCWindow(drawable){}
  FXPyDCWindow(FXDrawable* drawable,FXEvent* ev) : FXDCWindow(drawable,ev){}
  };

// Drawable
class FXPyDrawable : public FXDrawable {
  FXDECLARE(FXPyDrawable)
public:
  FXPyDrawable(){}
#include "object_virtuals.h"
  };


// Image
class FXPyImage : public FXImage {
  FXDECLARE(FXPyImage)
protected:
  FXPyImage(){}
public:
  FXPyImage(FXPyApp* a,const void *pix,FXuint opts,FXint w,FXint h) : FXImage(a,pix,opts,w,h){}
#include "object_virtuals.h"
  };


// Frame
class FXPyFrame : public FXFrame {
  FXDECLARE(FXPyFrame)
protected:
  FXPyFrame(){}
public:
  FXPyFrame(FXComposite* p,FXuint opts=FRAME_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint pb=DEFAULT_PAD) : FXFrame(p,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Horizontal Frame
class FXPyHorizontalFrame : public FXHorizontalFrame {
  FXDECLARE(FXPyHorizontalFrame)
protected:
  FXPyHorizontalFrame(){}
public:
  FXPyHorizontalFrame(FXComposite* p,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pb=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint hs=DEFAULT_SPACING,FXint vs=DEFAULT_SPACING) : FXHorizontalFrame(p,opts,x,y,w,h,pl,pr,pb,pt,hs,vs){}
#include "object_virtuals.h"
  };

// Horizontal Separator
class FXPyHorizontalSeparator : public FXHorizontalSeparator {
  FXDECLARE(FXPyHorizontalSeparator)
protected:
  FXPyHorizontalSeparator(){}
public:
  FXPyHorizontalSeparator(FXComposite* p,FXuint opts=SEPARATOR_GROOVE|LAYOUT_FILL_X,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=1,FXint pr=1,FXint pt=0,FXint pb=0) : FXHorizontalSeparator(p,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };

// Label
class FXPyLabel : public FXLabel {
  FXDECLARE(FXPyLabel)
protected:
  FXPyLabel(){}
public:
  FXPyLabel(FXComposite* p,const FXString& text,FXPyIcon *ic=0,FXuint opts=JUSTIFY_NORMAL|ICON_BEFORE_TEXT,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint pb=DEFAULT_PAD) : FXLabel(p,text,ic,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };

// Vertical Frame
class FXPyVerticalFrame : public FXVerticalFrame {
  FXDECLARE(FXPyVerticalFrame)
protected:
  FXPyVerticalFrame(){}
public:
  FXPyVerticalFrame(FXComposite* p,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pb=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint hs=DEFAULT_SPACING,FXint vs=DEFAULT_SPACING) : FXVerticalFrame(p,opts,x,y,w,h,pl,pr,pb,pt,hs,vs){}
#include "object_virtuals.h"
  };

// Vertical Separator
class FXPyVerticalSeparator : public FXVerticalSeparator {
  FXDECLARE(FXPyVerticalSeparator)
protected:
  FXPyVerticalSeparator(){}
public:
  FXPyVerticalSeparator(FXComposite* p,FXuint opts=SEPARATOR_GROOVE|LAYOUT_FILL_X,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=1,FXint pr=1,FXint pt=0,FXint pb=0) : FXVerticalSeparator(p,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Window
class FXPyWindow : public FXWindow {
  FXDECLARE(FXPyWindow)
protected:
  FXPyWindow(){}
  FXPyWindow(const FXPyWindow&);
public:
  FXPyWindow(FXComposite* p,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0) : FXWindow(p,opts,x,y,w,h){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Canvas
class FXPyCanvas : public FXCanvas {
  FXDECLARE(FXPyCanvas)
protected:
  FXPyCanvas(){}
public:
  FXPyCanvas(FXComposite* p,FXPyObject* tgt=NULL,FXSelector sel=0,FXuint opts=FRAME_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0) : FXCanvas(p,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Main window
class FXPyMainWindow : public FXMainWindow {
  FXDECLARE(FXPyMainWindow)
protected:
  FXPyMainWindow(){}
public:
  FXPyMainWindow(FXPyApp* a,const FXString& name,FXPyIcon *ic=NULL,FXPyIcon *mi=NULL,FXuint opts=DECOR_ALL,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint hs=4,FXint vs=4) : FXMainWindow(a,name,ic,mi,opts,x,y,w,h,hs,vs){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Packer
class FXPyPacker : public FXPacker {
  FXDECLARE(FXPyPacker)
protected:
  FXPyPacker(){}
public:
  FXPyPacker(FXComposite* p,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint pb=DEFAULT_PAD,FXint hs=DEFAULT_SPACING,FXint vs=DEFAULT_SPACING) : FXPacker(p,opts,x,y,w,h,pl,pr,pt,pb,hs,vs){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Popup
class FXPyPopup : public FXPopup {
  FXDECLARE(FXPyPopup)
protected:
  FXPyPopup(){}
public:
  FXPyPopup(FXPyWindow* owner,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXPopup(owner,opts,x,y,w,h) {}
#include "object_virtuals.h"
  };


// Shell
class FXPyShell : public FXShell {
  FXDECLARE(FXPyShell)
public:
  FXPyShell(){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Tooltip
class FXPyTooltip : public FXTooltip {
  FXDECLARE(FXPyTooltip)
protected:
  FXPyTooltip(){}
  FXPyTooltip(const FXPyTooltip&);
public:
  FXPyTooltip(FXPyApp* app,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0) : FXTooltip(app,opts,x,y,w,h){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };

// Top-level window
class FXPyTopWindow : public FXTopWindow {
  FXDECLARE(FXPyTopWindow)
public:
  FXPyTopWindow(){}
  FXPyTopWindow(FXPyWindow* owner,const FXString& name,FXPyIcon *ic,FXPyIcon *mi,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint hs,FXint vs) : FXTopWindow(owner,name,ic,mi,opts,x,y,w,h,hs,vs){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Dialog Box
class FXPyDialogBox : public FXDialogBox {
  FXDECLARE(FXPyDialogBox)
protected:
  FXPyDialogBox(){}
public:
  FXPyDialogBox(FXPyWindow* owner,const FXString& name,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint hs,FXint vs) : FXDialogBox(owner,name,opts,x,y,w,h,hs,vs){}
#include "object_virtuals.h"
  };


// Status bar
class FXPyStatusbar : public FXStatusbar {
  FXDECLARE(FXPyStatusbar)
protected:
  FXPyStatusbar(){}
public:
  FXPyStatusbar(FXPyComposite* p,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs) : FXStatusbar(p,opts,x,y,w,h,pl,pr,pt,pb,hs,vs){}
#include "object_virtuals.h"
  };


// Group box
class FXPyGroupBox : public FXGroupBox {
  FXDECLARE(FXPyGroupBox)
protected:
  FXPyGroupBox(){}
public:
  FXPyGroupBox(FXPyComposite* p,const FXString& text,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs) : FXGroupBox(p,text,opts,x,y,w,h,pl,pr,pt,pb,hs,vs){}
#include "object_virtuals.h"
  };


// Check button
class FXPyCheckButton : public FXCheckButton {
  FXDECLARE(FXPyCheckButton)
protected:
  FXPyCheckButton(){}
public:
  FXPyCheckButton(FXPyComposite* p,const FXString& text,FXPyObject *tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXCheckButton(p,text,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
  };


// Radio button
class FXPyRadioButton : public FXRadioButton {
  FXDECLARE(FXPyRadioButton)
protected:
  FXPyRadioButton(){}
public:
  FXPyRadioButton(FXPyComposite* p,const FXString& text,FXPyObject *tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXRadioButton(p,text,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
  };


// Menu item separator
class FXPyMenuSeparator : public FXMenuSeparator {
  FXDECLARE(FXPyMenuSeparator)
protected:
  FXPyMenuSeparator(){}
public:
  FXPyMenuSeparator(FXPyComposite* p,FXuint opts) : FXMenuSeparator(p,opts){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Text menu item
class FXPyMenuCaption : public FXMenuCaption {
  FXDECLARE(FXPyMenuCaption)
protected:
  FXPyMenuCaption(){}
public:
  FXPyMenuCaption(FXPyComposite* p,const FXString& text,FXPyIcon* ic,FXuint opts) : FXMenuCaption(p,text,ic,opts){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Popup menu pane
class FXPyMenuPane : public FXMenuPane {
  FXDECLARE(FXPyMenuPane)
protected:
  FXPyMenuPane(){}
public:
  FXPyMenuPane(FXPyWindow* owner,FXuint opts) : FXMenuPane(owner,opts){}
#include "object_virtuals.h"
  };


// Menu title button
class FXPyMenuTitle : public FXMenuTitle {
  FXDECLARE(FXPyMenuTitle)
protected:
  FXPyMenuTitle(){}
public:
  FXPyMenuTitle(FXPyComposite* p,const FXString& text,FXPyIcon* ic,FXPyPopup* pup,FXuint opts) : FXMenuTitle(p,text,ic,pup,opts){}
#include "object_virtuals.h"
  };


// Menu cascade button
class FXPyMenuCascade : public FXMenuCascade {
  FXDECLARE(FXPyMenuCascade)
protected:
  FXPyMenuCascade(){}
public:
  FXPyMenuCascade(FXPyComposite* p,const FXString& text,FXPyIcon* ic,FXPyMenuPane* pup,FXuint opts) : FXMenuCascade(p,text,ic,pup,opts){}
#include "object_virtuals.h"
  };


// Command menu item
class FXPyMenuCommand : public FXMenuCommand {
  FXDECLARE(FXPyMenuCommand)
protected:
  FXPyMenuCommand(){}
public:
  FXPyMenuCommand(FXPyComposite* p,const FXString& text,FXPyIcon* ic,FXPyObject* tgt,FXSelector sel,FXuint opts) : FXMenuCommand(p,text,ic,tgt,sel,opts){}
#include "object_virtuals.h"
  };
  

// Menu bar
class FXPyMenubar : public FXMenubar {
  FXDECLARE(FXPyMenubar)
protected:
  FXPyMenubar(){}
public:
  FXPyMenubar(FXPyComposite* p,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs) : FXMenuBar(p,opts,x,y,w,h,pl,pr,pt,pb,hs,vs){}
#include "object_virtuals.h"
  };


// Four-way split
class FXPy4Splitter : public FX4Splitter {
  FXDECLARE(FXPy4Splitter)
protected:
  FXPy4Splitter(){}
public:
  FXPy4Splitter(FXPyComposite* p,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FX4Splitter(p,opts,x,y,w,h) {}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Matrix packer
class FXPyMatrix : public FXMatrix {
  FXDECLARE(FXPyMatrix)
protected:
  FXPyMatrix(){}
public:
  FXPyMatrix(FXPyComposite* p,FXint n,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs) : FXMatrix(p,n,opts,x,y,w,h,pl,pr,pt,pb,hs,vs){}
#include "object_virtuals.h"
  };


// Dial
class FXPyDial : public FXDial {
  FXDECLARE(FXPyDial)
protected:
  FXPyDial(){}
public:
  FXPyDial(FXPyComposite *p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXDial(p,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Text field
class FXPyTextField : public FXTextField {
  FXDECLARE(FXPyTextField)
protected:
  FXPyTextField(){}
public:
  FXPyTextField(FXPyComposite* p,FXint cols,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXTextField(p,cols,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Slider
class FXPySlider : public FXSlider {
  FXDECLARE(FXPySlider)
protected:
  FXPySlider(){}
public:
  FXPySlider(FXPyComposite* p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXSlider(p,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Option button
class FXPyOption : public FXOption {
  FXDECLARE(FXPyOption)
protected:
  FXPyOption(){}
public:
  FXPyOption(FXPyComposite* p,const FXString& text,FXPyIcon* ic,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXOption(p,text,ic,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
  };


// Option menu
class FXPyOptionMenu : public FXOptionMenu {
  FXDECLARE(FXPyOptionMenu)
protected:
  FXPyOptionMenu(){}
public:
  FXPyOptionMenu(FXPyComposite* p,FXPyPopup* pup,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXOptionMenu(p,pup,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
  };


// Menu button
class FXPyMenuButton : public FXMenuButton {
  FXDECLARE(FXPyMenuButton)
protected:
  FXPyMenuButton(){}
public:
  FXPyMenuButton(FXPyComposite* p,const FXString& text,FXPyIcon* ic,FXPyPopup* pup,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXMenuButton(p,text,ic,pup,opts,x,y,w,h,pl,pr,pt,pb) {}
#include "object_virtuals.h"
  };


// Splitter
class FXPySplitter : public FXSplitter {
  FXDECLARE(FXPySplitter)
protected:
  FXPySplitter(){}
public:
  FXPySplitter(FXPyComposite* p,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXSplitter(p,opts,x,y,w,h){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Base class for scrolled stuff
class FXPyScrollArea : public FXScrollArea {
  FXDECLARE(FXPyScrollArea)
protected:
  FXPyScrollArea(){}
public:
  FXPyScrollArea(FXPyComposite* p,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXScrollArea(p,opts,x,y,w,h){}
#include "object_virtuals.h"
  };


// Automatic scroll area
class FXPyScrollWindow : public FXScrollWindow {
  FXDECLARE(FXPyScrollWindow)
protected:
  FXPyScrollWindow(){}
public:
  FXPyScrollWindow(FXPyComposite* p,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXScrollWindow(p,opts,x,y,w,h){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


class FXPyTreeList;


// Tree list item
class FXPyTreeItem : public FXTreeItem {
  FXDECLARE(FXPyTreeItem)
protected:
  FXPyTreeItem(){}
  FXPyTreeItem(const FXPyTreeItem&);
  FXPyTreeItem& operator=(const FXPyTreeItem&);
public:
  FXPyTreeItem(const FXString& text,FXIcon* oi,FXIcon *ci,void *ptr) : FXTreeItem(text,oi,ci,ptr){}
#include "object_virtuals.h"
#include "treeitem_virtuals.h"
};


// Tree list
class FXPyTreeList : public FXTreeList {
  FXDECLARE(FXPyTreeList)
protected:
  FXPyTreeList(){}
  FXPyTreeList(const FXPyTreeList&);
public:
  FXPyTreeList(FXPyComposite* p,FXint nvis,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXTreeList(p,nvis,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
#include "window_virtuals.h"
 };


// Tree list box
class FXPyTreeListBox : public FXTreeListBox {
  FXDECLARE(FXPyTreeListBox)
protected:
  FXPyTreeListBox(){}
  FXPyTreeListBox(const FXPyTreeListBox&);
public:
  FXPyTreeListBox(FXPyComposite* p,FXint nvis,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXTreeListBox(p,nvis,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
  };


// Toolbar Tab
class FXPyToolbarTab : public FXToolbarTab {
  FXDECLARE(FXPyToolbarTab)
protected:
  FXPyToolbarTab(){}
  FXPyToolbarTab(const FXPyToolbarTab&);
public:
  FXPyToolbarTab(FXPyComposite* p,FXPyObject* tgt=NULL,FXSelector sel=0,FXuint opts=FRAME_RAISED,FXint x=0,FXint y=0,FXint w=0,FXint h=0) : FXToolbarTab(p,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Tab Bar
class FXPyTabBar : public FXTabBar {
  FXDECLARE(FXPyTabBar)
protected:
  FXPyTabBar(){}
  FXPyTabBar(const FXPyTabBar&);
public:
  FXPyTabBar(FXPyComposite* p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXTabBar(p,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
  };


// Tab item
class FXPyTabItem : public FXTabItem {
  FXDECLARE(FXPyTabItem)
protected:
  FXPyTabItem(){}
  FXPyTabItem(const FXPyTabItem&);
public:
  FXPyTabItem(FXPyTabBar* p,const FXString& text,FXPyIcon* ic,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXTabItem(p,text,ic,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
  };


// Tab Book
class FXPyTabBook : public FXTabBook {
  FXDECLARE(FXPyTabBook)
protected:
  FXPyTabBook(){}
  FXPyTabBook(const FXPyTabBook&);
public:
  FXPyTabBook(FXPyComposite* p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXTabBook(p,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
  };


// Scroll bar
class FXPyScrollbar : public FXScrollbar {
  FXDECLARE(FXPyScrollbar)
protected:
  FXPyScrollbar(){}
  FXPyScrollbar(const FXPyScrollbar&);
public:
  FXPyScrollbar(FXPyComposite* p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXScrollbar(p,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Corner between scrollbars
class FXPyScrollCorner : public FXScrollCorner {
  FXDECLARE(FXPyScrollCorner)
protected:
  FXPyScrollCorner(){}
  FXPyScrollCorner(const FXPyScrollCorner&);
public:
  FXPyScrollCorner(FXPyComposite* p) : FXScrollCorner(p){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Progress bar widget
class FXPyProgressBar : public FXProgressBar {
  FXDECLARE(FXPyProgressBar)
protected:
  FXPyProgressBar(){}
  FXPyProgressBar(const FXPyProgressBar&);
public:
  FXPyProgressBar(FXPyComposite* p,FXPyObject* target,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXProgressBar(p,target,sel,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Printer selection
class FXPyPrintDialog : public FXPrintDialog {
  FXDECLARE(FXPyPrintDialog)
protected:
  FXPyPrintDialog(){}
  FXPyPrintDialog(const FXPyPrintDialog&);
public:
  FXPyPrintDialog(FXPyWindow* owner,const FXString& name,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXPrintDialog(owner,name,opts,x,y,w,h){}
#include "object_virtuals.h"
  };


class FXPyVisual : public FXVisual {
  FXDECLARE(FXPyVisual)
protected:
  FXPyVisual(){}
  FXPyVisual(const FXPyVisual&);
public:
  FXPyVisual(FXPyApp* a,FXuint flgs,FXuint d=32) : FXVisual(a,flgs,d){}
#include "object_virtuals.h"
  };


class FXPyBMPImage : public FXBMPImage {
  FXDECLARE(FXPyBMPImage)
protected:
  FXPyBMPImage(){}
  FXPyBMPImage(FXPyBMPImage&);
public:
  FXPyBMPImage(FXPyApp* a,const void *pix,FXuint opts,FXint w,FXint h) : FXBMPImage(a,pix,opts,w,h){}
#include "object_virtuals.h"
  };


class FXPyGIFImage : public FXGIFImage {
  FXDECLARE(FXPyGIFImage)
protected:
  FXPyGIFImage(){}
  FXPyGIFImage(FXPyGIFImage&);
public:
  FXPyGIFImage(FXPyApp* a,const void *pix,FXuint opts,FXint w,FXint h) : FXGIFImage(a,pix,opts,w,h){}
#include "object_virtuals.h"
  };


class FXPyPNGImage : public FXPNGImage {
  FXDECLARE(FXPyPNGImage)
protected:
  FXPyPNGImage(){}
  FXPyPNGImage(FXPyPNGImage&);
public:
  FXPyPNGImage(FXPyApp* a,const void *pix,FXuint opts,FXint w,FXint h) : FXPNGImage(a,pix,opts,w,h){}
#include "object_virtuals.h"
  };


class FXPyColorWell : public FXColorWell {
  FXDECLARE(FXPyColorWell)
protected:
  FXPyColorWell(){}
  FXPyColorWell(FXPyColorWell&);
public:
  FXPyColorWell(FXPyComposite* p,FXColor clr,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXColorWell(p,clr,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


class FXPyArrowButton : public FXArrowButton {
  FXDECLARE(FXPyArrowButton)
protected:
  FXPyArrowButton(){}
  FXPyArrowButton(FXPyArrowButton&);
public:
  FXPyArrowButton(FXPyComposite* p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXArrowButton(p,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


class FXPyToggleButton : public FXToggleButton {
  FXDECLARE(FXPyToggleButton)
protected:
  FXPyToggleButton(){}
  FXPyToggleButton(FXPyToggleButton&);
public:
  FXPyToggleButton(FXPyComposite* p,const FXString& text1,const FXString& text2,FXPyIcon* ic1,FXPyIcon* ic2,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXToggleButton(p,text1,text2,ic1,ic2,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


class FXPyRootWindow : public FXRootWindow {
  FXDECLARE(FXPyRootWindow)
protected:
  FXPyRootWindow(){}
  FXPyRootWindow(FXPyRootWindow&);
public:
  FXPyRootWindow(FXPyApp* a,FXPyVisual* vis) : FXRootWindow(a,vis){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


class FXPySpinner : public FXSpinner {
  FXDECLARE(FXPySpinner)
protected:
  FXPySpinner(){}
  FXPySpinner(FXPySpinner&);
public:
  FXPySpinner(FXPyComposite *p,FXint cols,FXPyObject *tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXSpinner(p,cols,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
  };


class FXPySwitcher : public FXSwitcher {
  FXDECLARE(FXPySwitcher)
protected:
  FXPySwitcher(){}
  FXPySwitcher(FXPySwitcher&);
public:
  FXPySwitcher(FXPyComposite *p,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXSwitcher(p,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
  };


class FXPyList : public FXList {
  FXDECLARE(FXPyList)
protected:
  FXPyList(){}
  FXPyList(FXPyList&);
public:
  FXPyList(FXPyComposite *p,FXint nvis,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXList(p,nvis,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
  };


class FXPyListItem : public FXListItem {
  FXDECLARE(FXPyListItem)
protected:
  FXPyListItem(){}
  FXPyListItem(const FXPyListItem&);
public:
  FXPyListItem(const FXString& text,FXPyIcon* ic,void* ptr) : FXListItem(text,ic,ptr){}
#include "object_virtuals.h"
#include "listitem_virtuals.h"
  };


class FXPyComboBox : public FXComboBox {
  FXDECLARE(FXPyComboBox)
protected:
  FXPyComboBox(){}
  FXPyComboBox(FXPyComboBox&);
public:
  FXPyComboBox(FXPyComposite *p,FXint cols,FXint nvis,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXComboBox(p,cols,nvis,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
  };


class FXPyMessageBox : public FXMessageBox {
  FXDECLARE(FXPyMessageBox)
protected:
  FXPyMessageBox(){}
  FXPyMessageBox(FXPyMessageBox&);
public:
  FXPyMessageBox(FXPyWindow* owner,const FXString& name,const FXString& text,FXPyIcon* ic,FXuint opts,FXint x,FXint y) : FXMessageBox(owner,name,text,ic,opts,x,y){}
#include "object_virtuals.h"
  };


// Directory list items
class FXPyDirItem : public FXDirItem {
  FXDECLARE(FXPyDirItem)
protected:
  FXPyDirItem(){}
  FXPyDirItem(const FXPyDirItem&);
  FXPyDirItem& operator=(const FXPyDirItem&);
public:
  FXPyDirItem(const FXString& text,FXPyIcon* oi,FXPyIcon* ci,void* ptr) : FXDirItem(text,oi,ci,ptr) {}
#include "object_virtuals.h"
#include "treeitem_virtuals.h"
};


class FXPyDirList : public FXDirList {
  FXDECLARE(FXPyDirList)
protected:
  FXPyDirList(){}
  FXPyDirList(FXPyDirList&);
public:
  FXPyDirList(FXPyComposite *p,FXint nvis,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXDirList(p,nvis,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
  };


class FXPyHeader;


class FXPyHeaderItem : public FXHeaderItem {
  FXDECLARE(FXPyHeaderItem)
protected:
  FXPyHeaderItem(){}
  FXPyHeaderItem(const FXPyHeaderItem&);
public:
  FXPyHeaderItem(const FXString& text,FXPyIcon *ic,FXint size,void *ptr) : FXHeaderItem(text,ic,size,ptr){}
#include "object_virtuals.h"
#include "headeritem_virtuals.h"
  };


class FXPyHeader : public FXHeader {
  FXDECLARE(FXPyHeader)
protected:
  FXPyHeader(){}
  FXPyHeader(FXPyHeader&);
public:
  FXPyHeader(FXPyComposite *p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXHeader(p,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


class FXPyShutter : public FXShutter {
  FXDECLARE(FXPyShutter)
protected:
  FXPyShutter(){}
  FXPyShutter(FXPyShutter&);
public:
  FXPyShutter(FXPyComposite *p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs) : FXShutter(p,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb,hs,vs){}
#include "object_virtuals.h"
  };


class FXPyShutterItem : public FXShutterItem {
  FXDECLARE(FXPyShutterItem)
protected:
  FXPyShutterItem(){}
  FXPyShutterItem(const FXPyShutterItem&);
public:
  FXPyShutterItem(FXPyShutter *p,const char* text,FXPyIcon* ic,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs) : FXShutterItem(p,text,ic,opts,x,y,w,h,pl,pr,pt,pb,hs,vs){}
#include "object_virtuals.h"
  };


class FXPyIconList;


class FXPyIconItem : public FXIconItem {
  FXDECLARE(FXPyIconItem)
protected:
  FXPyIconItem(){}
  FXPyIconItem(const FXPyIconItem&);
public:
  FXPyIconItem(const FXString& text,FXPyIcon* bi,FXPyIcon* mi,void* ptr) : FXIconItem(text,bi,mi,ptr){}
#include "object_virtuals.h"
#include "iconitem_virtuals.h"
  };


class FXPyIconList : public FXIconList {
  FXDECLARE(FXPyIconList)
protected:
  FXPyIconList(){}
  FXPyIconList(FXPyIconList&);
public:
  FXPyIconList(FXPyComposite *p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXIconList(p,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
  };


class FXPyFileItem : public FXFileItem {
  FXDECLARE(FXPyFileItem)
protected:
  FXPyFileItem(){}
  FXPyFileItem(const FXPyFileItem&);
  FXPyFileItem& operator=(const FXPyFileItem&);
public:
  FXPyFileItem(const FXString& text,FXPyIcon* bi,FXPyIcon* mi,void* ptr) : FXFileItem(text,bi,mi,ptr) {}
#include "object_virtuals.h"
#include "iconitem_virtuals.h"
};


class FXPyFileList : public FXFileList {
  FXDECLARE(FXPyFileList)
protected:
  FXPyFileList(){}
  FXPyFileList(const FXPyFileList&);
public:
  FXPyFileList(FXPyComposite *p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXFileList(p,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
  };


class FXPyDirBox : public FXDirBox {
  FXDECLARE(FXPyDirBox)
protected:
  FXPyDirBox(){}
  FXPyDirBox(FXPyDirBox&);
public:
  FXPyDirBox(FXPyComposite *p,FXint nvis,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXDirBox(p,nvis,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
  };


class FXPyFileSelector : public FXFileSelector {
  FXDECLARE(FXPyFileSelector)
protected:
  FXPyFileSelector(){}
  FXPyFileSelector(FXPyFileSelector&);
public:
  FXPyFileSelector(FXPyComposite *p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXFileSelector(p,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
  };


class FXPyFileDialog : public FXFileDialog {
  FXDECLARE(FXPyFileDialog)
protected:
  FXPyFileDialog(){}
  FXPyFileDialog(FXPyFileDialog&);
public:
  FXPyFileDialog(FXPyWindow* owner,const FXString& name,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXFileDialog(owner,name,opts,x,y,w,h){}
#include "object_virtuals.h"
  };


class FXPyColorSelector : public FXColorSelector {
  FXDECLARE(FXPyColorSelector)
protected:
  FXPyColorSelector(){}
  FXPyColorSelector(FXPyColorSelector&);
public:
  FXPyColorSelector(FXPyComposite *p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXColorSelector(p,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
  };


class FXPyColorDialog : public FXColorDialog {
  FXDECLARE(FXPyColorDialog)
protected:
  FXPyColorDialog(){}
  FXPyColorDialog(FXPyColorDialog&);
public:
  FXPyColorDialog(FXPyWindow* owner,const FXString& name,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXColorDialog(owner,name,opts,x,y,w,h){}
#include "object_virtuals.h"
  };


class FXPyFontSelector : public FXFontSelector {
  FXDECLARE(FXPyFontSelector)
protected:
  FXPyFontSelector(){}
  FXPyFontSelector(FXPyFontSelector&);
public:
  FXPyFontSelector(FXPyComposite *p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXFontSelector(p,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
  };


class FXPyFontDialog : public FXFontDialog {
  FXDECLARE(FXPyFontDialog)
protected:
  FXPyFontDialog(){}
  FXPyFontDialog(FXPyFontDialog&);
public:
  FXPyFontDialog(FXPyWindow* owner,const FXString& name,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXFontDialog(owner,name,opts,x,y,w,h){}
#include "object_virtuals.h"
  };


class FXPyText : public FXText {
  FXDECLARE(FXPyText)
protected:
  FXPyText(){}
  FXPyText(FXPyText&);
public:
  FXPyText(FXPyComposite *p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXText(p,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
  };


class FXPyDocument : public FXDocument {
  FXDECLARE(FXPyDocument)
protected:
  FXPyDocument(FXPyDocument&);
public:
  FXPyDocument() : FXDocument(){}
#include "object_virtuals.h"
  };


class FXPyMDIDeleteButton : public FXMDIDeleteButton {
  FXDECLARE(FXPyMDIDeleteButton)
protected:
  FXPyMDIDeleteButton(){}
  FXPyMDIDeleteButton(FXPyMDIDeleteButton&);
public:
  FXPyMDIDeleteButton(FXPyComposite* p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXMDIDeleteButton(p,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
  };


class FXPyMDIMaximizeButton : public FXMDIMaximizeButton {
  FXDECLARE(FXPyMDIMaximizeButton)
protected:
  FXPyMDIMaximizeButton(){}
  FXPyMDIMaximizeButton(FXPyMDIMaximizeButton&);
public:
  FXPyMDIMaximizeButton(FXPyComposite* p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXMDIMaximizeButton(p,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
  };


class FXPyMDIMenu : public FXMDIMenu {
  FXDECLARE(FXPyMDIMenu)
protected:
  FXPyMDIMenu(){}
  FXPyMDIMenu(FXPyMDIMenu&);
public:
  FXPyMDIMenu(FXPyWindow* owner,FXPyObject* tgt) : FXMDIMenu(owner,tgt){}
#include "object_virtuals.h"
  };


class FXPyMDIMinimizeButton : public FXMDIMinimizeButton {
  FXDECLARE(FXPyMDIMinimizeButton)
protected:
  FXPyMDIMinimizeButton(){}
  FXPyMDIMinimizeButton(FXPyMDIMinimizeButton&);
public:
  FXPyMDIMinimizeButton(FXPyComposite* p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXMDIMinimizeButton(p,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
  };


class FXPyMDIRestoreButton : public FXMDIRestoreButton {
  FXDECLARE(FXPyMDIRestoreButton)
protected:
  FXPyMDIRestoreButton(){}
  FXPyMDIRestoreButton(FXPyMDIRestoreButton&);
public:
  FXPyMDIRestoreButton(FXPyComposite* p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXMDIRestoreButton(p,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
  };


class FXPyMDIWindowButton : public FXMDIWindowButton {
  FXDECLARE(FXPyMDIWindowButton)
protected:
  FXPyMDIWindowButton(){}
  FXPyMDIWindowButton(FXPyMDIWindowButton&);
public:
  FXPyMDIWindowButton(FXPyComposite* p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXMDIWindowButton(p,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
  };


class FXPyMDIClient : public FXMDIClient {
  FXDECLARE(FXPyMDIClient)
protected:
  FXPyMDIClient(){}
  FXPyMDIClient(FXPyMDIClient&);
public:
  FXPyMDIClient(FXPyComposite* p,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXMDIClient(p,opts,x,y,w,h){}
#include "object_virtuals.h"
  };


class FXPyMDIChild : public FXMDIChild {
  FXDECLARE(FXPyMDIChild)
protected:
  FXPyMDIChild(){}
  FXPyMDIChild(FXPyMDIChild&);
public:
  FXPyMDIChild(FXPyMDIClient* p,const FXString& name,FXPyIcon* ic,FXPyMenuPane* mn,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXMDIChild(p,name,ic,mn,opts,x,y,w,h){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };

// GL object
class FXPyGLObject : public FXGLObject {
  FXDECLARE(FXPyGLObject)
public:
  FXPyGLObject() : FXGLObject() {}
#include "object_virtuals.h"
#include "globject_virtuals.h"
  };


// Shape
class FXPyGLShape : public FXGLShape {
  FXDECLARE(FXPyGLShape)
protected:
  FXPyGLShape(){}
  FXPyGLShape(const FXPyGLShape&);
  FXPyGLShape& operator=(const FXPyGLShape&);
public:
  FXPyGLShape(FXfloat x,FXfloat y,FXfloat z,FXuint opts) : FXGLShape(x,y,z,opts) {}
#include "object_virtuals.h"
#include "globject_virtuals.h"
};


// Cone
class FXPyGLCone : public FXGLCone {
  FXDECLARE(FXPyGLCone)
protected:
  FXPyGLCone(){}
  FXPyGLCone(const FXPyGLCone&);
  FXPyGLCone& operator=(const FXPyGLCone&);
public:
  FXPyGLCone(FXfloat x,FXfloat y,FXfloat z,FXfloat h,FXfloat r) : FXGLCone(x,y,z,h,r){}
#include "object_virtuals.h"
#include "globject_virtuals.h"
};


// Cylinder
class FXPyGLCylinder : public FXGLCylinder {
  FXDECLARE(FXPyGLCylinder)
protected:
  FXPyGLCylinder(){}
  FXPyGLCylinder(const FXPyGLCylinder&);
  FXPyGLCylinder& operator=(const FXPyGLCylinder&);
public:
  FXPyGLCylinder(FXfloat x,FXfloat y,FXfloat z,FXfloat h,FXfloat r) : FXGLCylinder(x,y,z,h,r){}
#include "object_virtuals.h"
#include "globject_virtuals.h"
};


// Triangle Mesh
class FXPyGLTriangleMesh : public FXGLTriangleMesh {
  FXDECLARE(FXPyGLTriangleMesh)
protected:
  FXPyGLTriangleMesh(){}
  FXPyGLTriangleMesh(const FXPyGLTriangleMesh&);
  FXPyGLTriangleMesh& operator=(const FXPyGLTriangleMesh&);
public:
  FXPyGLTriangleMesh(FXfloat x,FXfloat y,FXfloat z,FXint nv,FXfloat* v,FXfloat *c,FXfloat* p,FXfloat* t) : FXGLTriangleMesh(x,y,z,nv,v,c,p,t) {}
#include "object_virtuals.h"
#include "globject_virtuals.h"
};


// OpenGL Visual
class FXPyGLVisual : public FXGLVisual {
  FXDECLARE(FXPyGLVisual)
protected:
  FXPyGLVisual(){}
  FXPyGLVisual(FXPyGLVisual&);
public:
  FXPyGLVisual(FXPyApp* app,FXuint flags) : FXGLVisual(app,flags){}
#include "object_virtuals.h"
  };


class FXPyGLCanvas : public FXGLCanvas {
  FXDECLARE(FXPyGLCanvas)
protected:
  FXPyGLCanvas(){}
  FXPyGLCanvas(FXPyGLCanvas&);
public:
  FXPyGLCanvas(FXPyComposite* p,FXPyGLVisual *vis,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXGLCanvas(p,vis,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


class FXPyGLCube : public FXGLCube {
  FXDECLARE(FXPyGLCube)
protected:
  FXPyGLCube(){}
  FXPyGLCube(FXPyGLCube&);
public:
  FXPyGLCube(FXfloat x,FXfloat y,FXfloat z,FXfloat w,FXfloat h,FXfloat d) : FXGLCube(x,y,z,w,h,d){}
#include "object_virtuals.h"
#include "globject_virtuals.h"
 };


class FXPyGLGroup : public FXGLGroup {
  FXDECLARE(FXPyGLGroup)
protected:
  FXPyGLGroup(FXPyGLGroup&);
public:
  FXPyGLGroup() : FXGLGroup(){}
#include "object_virtuals.h"
#include "globject_virtuals.h"
  };


class FXPyGLLine : public FXGLLine {
  FXDECLARE(FXPyGLLine)
protected:
  FXPyGLLine(){}
  FXPyGLLine(FXPyGLLine&);
public:
  FXPyGLLine(FXfloat fx,FXfloat fy,FXfloat fz,FXfloat tx,FXfloat ty,FXfloat tz) : FXGLLine(fx,fy,fz,tx,ty,tz){}
#include "object_virtuals.h"
#include "globject_virtuals.h"
  };


class FXPyGLPoint : public FXGLPoint {
  FXDECLARE(FXPyGLPoint)
protected:
  FXPyGLPoint(){}
  FXPyGLPoint(FXPyGLPoint&);
public:
  FXPyGLPoint(FXfloat x,FXfloat y,FXfloat z) : FXGLPoint(x,y,z){}
#include "object_virtuals.h"
#include "globject_virtuals.h"
  };


class FXPyGLSphere : public FXGLSphere {
  FXDECLARE(FXPyGLSphere)
protected:
  FXPyGLSphere(){}
  FXPyGLSphere(FXPyGLSphere&);
public:
  FXPyGLSphere(FXfloat x,FXfloat y,FXfloat z,FXfloat r) : FXGLSphere(x,y,z,r){}
#include "object_virtuals.h"
#include "globject_virtuals.h"
  };


class FXPyGLViewer : public FXGLViewer {
  FXDECLARE(FXPyGLViewer)
protected:
  FXPyGLViewer(){}
  FXPyGLViewer(FXPyGLViewer&);
public:
  FXPyGLViewer(FXPyComposite* p,FXPyGLVisual *vis,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXGLViewer(p,vis,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Table
class FXPyTable : public FXTable {
  FXDECLARE(FXPyTable)
protected:
  FXPyTable(){}
  FXPyTable(const FXPyTable&);
  FXPyTable& operator=(const FXPyTable& rhs);
public:
  FXPyTable(FXComposite *p,FXint nr,FXint nc,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) : FXTable(p,nr,nc,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Table items
class FXPyTableItem : public FXTableItem {
  FXDECLARE(FXPyTableItem)
protected:
  FXPyTableItem(){}
  FXPyTableItem(const FXPyTableItem&);
  FXPyTableItem& operator=(const FXPyTableItem& rhs);
public:
  FXPyTableItem(const FXString& text,FXIcon *ic,void *ptr) : FXTableItem(text,ic,ptr) {}
#include "object_virtuals.h"
#include "tableitem_virtuals.h"
};


// Monochrome bitmap
class FXPyBitmap : public FXBitmap {
  FXDECLARE(FXPyBitmap)
protected:
  FXPyBitmap(){}
  FXPyBitmap(const FXPyBitmap&);
public:
  FXPyBitmap(FXPyApp* app,const void* pix,FXuint opts,FXint w,FXint h) : FXBitmap(app,pix,opts,w,h){}
#include "object_virtuals.h"
  };


// Dictionary
class FXPyDict : public FXDict {
  FXDECLARE(FXPyDict)
public:
  FXPyDict(){}
#include "object_virtuals.h"
  };


// Icon dictionary
class FXPyIconDict : public FXIconDict {
  FXDECLARE(FXPyIconDict)
protected:
  FXPyIconDict(){}
  FXPyIconDict(const FXPyIconDict&);
public:
  FXPyIconDict(FXPyApp* app) : FXIconDict(app) {}
#include "object_virtuals.h"
  };


// File dictionary
class FXPyFileDict : public FXFileDict {
  FXDECLARE(FXPyFileDict)
protected:
  FXPyFileDict(){}
  FXPyFileDict(const FXPyFileDict&);
public:
  FXPyFileDict(FXPyApp* app) : FXFileDict(app) {}
#include "object_virtuals.h"
  };


// String dictionary
class FXPyStringDict : public FXStringDict {
  FXDECLARE(FXPyStringDict)
protected:
  FXPyStringDict(const FXPyStringDict&);
public:
  FXPyStringDict(){}
#include "object_virtuals.h"
  };


// Section dictionary
class FXPySectionDict : public FXSectionDict {
  FXDECLARE(FXPySectionDict)
protected:
  FXPySectionDict(const FXPySectionDict&);
public:
  FXPySectionDict(){}
#include "object_virtuals.h"
  };


// Status line
class FXPyStatusline : public FXStatusline {
  FXDECLARE(FXPyStatusline)
protected:
  FXPyStatusline(){}
  FXPyStatusline(const FXPyStatusline&);
public:
  FXPyStatusline(FXPyComposite* p,FXPyObject* tgt,FXSelector sel) : FXStatusline(p,tgt,sel) {}
#include "object_virtuals.h"
#include "window_virtuals.h"
};


// Drag corner for same
class FXPyDragCorner : public FXDragCorner {
  FXDECLARE(FXPyDragCorner)
protected:
  FXPyDragCorner(){}
  FXPyDragCorner(const FXPyDragCorner&);
public:
  FXPyDragCorner(FXPyComposite* p) : FXDragCorner(p) {}
#include "object_virtuals.h"
#include "window_virtuals.h"
};


// Recently used files
class FXPyRecentFiles : public FXRecentFiles {
  FXDECLARE(FXPyRecentFiles)
private:
  FXPyRecentFiles(const FXPyRecentFiles&);
  FXPyRecentFiles& operator=(const FXPyRecentFiles&);
public:
    FXPyRecentFiles() : FXRecentFiles() {}
#include "object_virtuals.h"
  };


/// X Pixmap icon
class FXPyXPMIcon : public FXXPMIcon {
  FXDECLARE(FXPyXPMIcon)
private:
  FXPyXPMIcon(const FXPyXPMIcon&);
  FXPyXPMIcon &operator=(const FXPyXPMIcon&);
protected:
  FXPyXPMIcon(){}
public:
  /// Construct icon from compiled-in X Pixmap format
  FXPyXPMIcon(FXPyApp* a,const FXchar **pix=NULL,FXColor clr=0,FXuint opts=0,FXint w=1,FXint h=1) : FXXPMIcon(a,pix,clr,opts,w,h){}
  };


/// X Pixmap image
class FXPyXPMImage : public FXXPMImage {
  FXDECLARE(FXPyXPMImage)
protected:
  FXPyXPMImage(){}
private:
  FXPyXPMImage(const FXPyXPMImage&);
  FXPyXPMImage &operator=(const FXPyXPMImage&);
public:
  /// Construct image from compiled-in X Pixmap format
  FXPyXPMImage(FXPyApp* a,const FXchar **pix=NULL,FXuint opts=0,FXint w=1,FXint h=1) : FXXPMImage(a,pix,opts,w,h){}
  };


/// Delegator
class FXPyDelegator : public FXDelegator {
  FXDECLARE(FXPyDelegator)
private:
  FXPyDelegator(const FXPyDelegator&);
  FXPyDelegator& operator=(const FXPyDelegator&);
public:
  FXPyDelegator(FXPyObject* target=NULL) : FXDelegator(target){}
#include "object_virtuals.h"
  };


/// Image view
class FXPyImageView : public FXImageView {
  FXDECLARE(FXPyImageView)
private:
  FXPyImageView(const FXPyImageView&);
  FXPyImageView& operator=(const FXPyImageView&);
protected:
  FXPyImageView(){}
public:
  FXPyImageView(FXPyComposite* p,FXPyImage* img=NULL,FXPyObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0) : FXImageView(p,img,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Toolbar
class FXPyToolbar : public FXToolbar {
  FXDECLARE(FXPyToolbar)
private:
  FXPyToolbar(const FXPyToolbar&);
  FXPyToolbar& operator=(const FXPyToolbar&);
protected:
  FXPyToolbar(){}
public:
  FXPyToolbar(FXPyComposite* p,FXPyComposite* q,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb,FXint hs,FXint vs) : FXToolbar(p,q,opts,x,y,w,h,pl,pr,pt,pb,hs,vs){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Toolbar Shell
class FXPyToolbarShell : public FXToolbarShell {
  FXDECLARE(FXPyToolbarShell)
private:
  FXPyToolbarShell(const FXPyToolbarShell&);
  FXPyToolbarShell& operator=(const FXPyToolbarShell&);
protected:
  FXPyToolbarShell(){}
public:
  FXPyToolbarShell(FXPyWindow* owner,FXuint opts=FRAME_RAISED|FRAME_THICK,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint hs=4,FXint vs=4) : FXToolbarShell(owner,opts,x,y,w,h,hs,vs){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };

// Directory selection dialog
class FXPyDirDialog : public FXDirDialog {
  FXDECLARE(FXPyDirDialog)
private:
  FXPyDirDialog(const FXPyDirDialog&);
  FXPyDirDialog& operator=(const FXPyDirDialog&);
protected:
  FXPyDirDialog(){}
public:
  FXPyDirDialog(FXPyWindow* owner,const FXString& name,FXuint opts=0,FXint x=0,FXint y=0,FXint w=500,FXint h=300) : FXDirDialog(owner,name,opts,x,y,w,h) {}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };

// Directory selection widget
class FXPyDirSelector : public FXDirSelector {
  FXDECLARE(FXPyDirSelector)
private:
  FXPyDirSelector(const FXPyDirSelector&);
  FXPyDirSelector& operator=(const FXPyDirSelector&);
protected:
  FXPyDirSelector(){}
public:
  FXPyDirSelector(FXPyComposite *p,FXPyObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0) : FXDirSelector(p,tgt,sel,opts,x,y,w,h) {}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Replace text
class FXPyReplaceDialog : public FXReplaceDialog {
  FXDECLARE(FXPyReplaceDialog)
private:
  FXPyReplaceDialog(const FXPyReplaceDialog&);
  FXPyReplaceDialog& operator=(const FXPyReplaceDialog&);
protected:
  FXPyReplaceDialog(){}
public:
  FXPyReplaceDialog(FXPyWindow* owner,const FXString& name,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0) : FXReplaceDialog(owner,name,opts,x,y,w,h) {}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// Search text
class FXPySearchDialog : public FXSearchDialog {
  FXDECLARE(FXPySearchDialog)
private:
  FXPySearchDialog(const FXPySearchDialog&);
  FXPySearchDialog& operator=(const FXPySearchDialog&);
protected:
  FXPySearchDialog(){}
public:
  FXPySearchDialog(FXPyWindow* owner,const FXString& name,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0) : FXSearchDialog(owner,name,opts,x,y,w,h) {}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };


// OpenGL Context
class FXPyGLContext : public FXGLContext {
  FXDECLARE(FXPyGLContext)
private:
  FXPyGLContext(const FXPyGLContext&);
  FXPyGLContext& operator=(const FXPyGLContext&);
protected:
  FXPyGLContext(){}
public:
  FXPyGLContext(FXPyApp* app,FXPyGLVisual* vis) : FXGLContext(app,vis) {}
#include "object_virtuals.h"
  };


class FXPyToolbarGrip : public FXToolbarGrip {
  FXDECLARE(FXPyToolbarGrip)
private:
  FXPyToolbarGrip(const FXPyToolbarGrip&);
  FXPyToolbarGrip& operator=(const FXPyToolbarGrip&);
protected:
  FXPyToolbarGrip(){}
public:
  FXPyToolbarGrip(FXPyToolbar* p,FXPyObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXToolbarGrip(p,tgt,sel,opts,x,y,w,h){}
#include "object_virtuals.h"
#include "window_virtuals.h"
  };

#endif
